#!/bin/sh

apt-get update
apt-get upgrade -y
apt-get install -y \
	build-essential \
	jq \
	libglib2.0-dev \
	libreadline-dev \
	libxml2 \
	libxml2-dev \
	nettle-dev \
	pkg-config \
	texinfo \
	valgrind \
	wget

export $(dbus-launch)

VERSION=$(cat dist/version)

cp dist/tarot-$VERSION.tar.gz . || exit 1
tar xf tarot-$VERSION.tar.gz || exit 1
mkdir build || exit 1
cd build/
../tarot-$VERSION/configure \
    --prefix="$PWD/../public" \
    --enable-silent-rules=yes \
    --enable-valgrind=yes \
    --disable-program \
    CFLAGS="-g -fprofile-arcs -ftest-coverage" \
    || (cat config.log; exit 1)
make -j 16 install-html || exit 1
make distclean || exit 1
cd .. || exit 1

cp tarot-$VERSION/index.html public/ || exit 1

if test "x$CI_PAGES_URL" = "x"
then
    export CI_PAGES_URL="https://play-tarot.frama.io/tarot"
fi

for link in $(wget -O- "$CI_API_V4_URL/projects/$CI_PROJECT_ID/releases" | jq -r '.[0].assets.links[].url')
do
    case $link in
	*/tarot-*.*.*.tar.gz)
	    wget -O public/tarot-latest.tar.gz "$link" || exit 1;;
	*)
	    echo "Unknown link '$link'" ; exit 1;;
    esac
done
